/*
 Standalone Current Logger for ATtiny85
 Version: 0.1
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 25/04/2012
 Last Modified: 25/04/2012
 
 This is a test to use an ATtiny85 with an OPA333 op-amp to log the current passing through a load.
 Logs 150 readings in under 20ms and transfers it to your PC. Needs to be powered by USB.
 Press the button to begin.
 
 Remember to change your fuse bits so the ATtiny85 uses 16MHz PLL and divide clock by 8
 avrdude -p ATtiny85 -c usbtiny -U lfuse:w:0x61:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m 
 
 */

// ATtiny85 Pin map
//                  +-\/-+
// Ain0 (D 5) PB5  1|o   |8  Vcc
// Ain3 (D 3) PB3  2|    |7  PB2 (D 2) Ain1
// Ain2 (D 4) PB4  3|    |6  PB1 (D 1) pwm1
//            GND  4|    |5  PB0 (D 0) pwm0
//                  +----+

#define adcPin 3 // Analog input 3
#define ledPin PB0 // Output
#define buttonPin PB4 // Input
#define usbminusPin PB1 // USB D-

#define F_CPU 16500000 // If this will be changed, modify it in usbconfig.h too

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "setup.c"

int adcMemory[150];

// Convert a number to HID values and add it to the nextDigit variable
void numberTohid(int number) {
	uchar digit;
	do {
		digit = number % 10;
		number /= 10;
		*--nextDigit = 0;
		if (digit == 0) {
			*--nextDigit = KEY_0;
		}
		else {
			*--nextDigit = KEY_1 - 1 + digit;
		}
	} while (number != 0);
}

// Send the next digit from the address
int sendnextDigit(uint16_t eepromAddress) {
	int value = adcMemory[eepromAddress];
	int baseline = 13; // 13 * 0.001074 = ~14mV baseline
	double calculateDouble = ((double) (value - baseline) * 0.01074);

	// Split the value into 2 parts
	int beforeDecimalpoint = (int) calculateDouble; // Before the decimal point
	int afterDecimalpoint = (int) ((calculateDouble - beforeDecimalpoint) * 1000); // After the decimal point
	
	// Fixes incorrect rounding after decimal point
	bool insertNumberzero = false;
	if (afterDecimalpoint < 100) {
		insertNumberzero = true;
	}
	
	nextDigit = &valueBuffer[sizeof(valueBuffer)];
	*--nextDigit = 0xff;
	*--nextDigit = 0;
	*--nextDigit = KEY_RETURN;
	
	if (afterDecimalpoint != 0) {
		numberTohid(afterDecimalpoint);
		
		if (insertNumberzero == true) {
			*--nextDigit = 0;
			*--nextDigit = KEY_0;
		}
		
		// Dot
		*--nextDigit = 0;
		*--nextDigit = KEY_DOT;
	}

	// First part
	numberTohid(beforeDecimalpoint);
	
	return 0;
}

// Transfers the data to USB
void transferData (void) {
	boolean inusbTransfer = true;
	uint16_t readCount = 0;
	uint16_t endCount = 150;
	
	while (inusbTransfer == true) {
		usbPoll();
		if(usbInterruptIsReady() && nextDigit != NULL){ // We can send another digit
			buildReport();
			usbSetInterrupt(reportBuffer, sizeof(reportBuffer));
			if(*++nextDigit == 0xff) { // This was terminator character
				nextDigit = NULL;
			}
		}
		if (nextDigit == NULL) {
			if (readCount < endCount) {
				int checkforZero = sendnextDigit(readCount);
				if (checkforZero == true) { // End if 0
					inusbTransfer = false;
				}
				readCount++;
			}
			else {
				inusbTransfer = false;
			}
		}
	}
}

int main (void) {
	setup();
	
	DDRB |= (1<<ledPin); // Set Outputs
	DDRB &= ~((1<<buttonPin) | (1<<usbminusPin)); // Set Inputs
	
	// Pin interrupt setup
	sbi(GIMSK,PCIE); // Enable pin change interrupt
	sbi(PCMSK,PCINT4); // Apply interrupt to button pin
	
	while(1) {			
		system_sleep();
		
		int adcResult = 0;
		int adcCount = 0;
		uint8_t adcLowbyte, adcHighbyte;
		
		// Turn on LED and ADC
		PORTB |= (1<<ledPin);
		sbi(ADCSRA, ADEN);
		sbi(ADCSRA, ADIE);
		
		// Start the ADC conversion
		sbi(ADCSRA, ADSC);
		
		// Wait to be woken up by the ADC interrupt
		set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
		sleep_mode(); // System sleeps here
		
		// Log ADC 150 times
		while (adcCount < 150) {
			// Read ADC bytes
			adcLowbyte = ADCL;
			adcHighbyte = ADCH;
			
			// Combine the bytes
			adcResult = (adcHighbyte << 8) | adcLowbyte;
			
			// Start the ADC conversion
			sbi(ADCSRA, ADSC);
			
			// Write the previous ADC result to memory
			adcMemory[adcCount] = adcResult;
			adcCount++;
			
			// Wait to be woken up by the ADC interrupt
			set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
			sleep_mode(); // System sleeps here
		}
		
		// Turn off LED and ADC
		PORTB &= ~(1<<ledPin);
		cbi(ADCSRA, ADSC);
		cbi(ADCSRA, ADEN);
		cbi(ADCSRA, ADIE);
		
		// Increase clock speed to 16MHz
		CLKPR = (1<<CLKPCE); // Prescaler enable
		CLKPR = 0; // Clock division factor 1 (0000)
		
		// Transfer the data to the PC
		startUSB();
		transferData();
		
		// Decrease clock speed to 1MHz
		CLKPR = (1<<CLKPCE); // Prescaler enable
		CLKPR = (1<<CLKPS2); // Clock division factor 16 (0100)
	}
	
	return 0;
}

// Pin change interrupt
ISR(PCINT0_vect) { }

// ADC conversion complete interrupt
ISR(ADC_vect) { }
